local Act = {}
local abs, exp = math.abs, math.exp

local function clip(x)
    if x < 0.01 then return 0.01 elseif x > 0.99 then x = 0.99 end
    return x
end

local function clip_sym(x)
    if x < -0.98 then x = -0.98 elseif x > 0.98 then x = 0.98 end
    return x
end

Act.sigmoid = {
    activation = function(self, x) return 1 / (1 + exp(-2 * x))            end,
    derivative = function(self, y) 
        y = clip(y)
        return 2 * y * ( 1 - y )
    end,
}

Act.elliot = {
    activation = function(self, x) return (x / 2) / (1 + abs(x) ) + 0.5    end,
    derivative = function(self, y)
        local t = 1 + abs( clip(y) )
        return 1 / (2 * t * t)
    end,
}

Act.threshold = {
    activation = function(self, x) return x > 0.5 and 1 or 0               end,
    derivative = function(self, y) error('Can\'t be used in training')     end, 
}

Act.linear = {
    activation = function(self, x) return x                                end,
    derivative = function(self, y) return 1                                end,
}

Act.sigmoid_symmetric = {
    activation = function(self, x) return 2 / (1 + exp(- 2 * x)) - 1       end,
    derivative = function(self, y)
        y = clip_sym(y)
        return 1 - ( y * y )
    end,
    msef       = 0.5,
    
}

Act.elliot_symmetric = {
    activation = function(self, x) return x / (1 + abs(x))                 end,
    derivative = function(self, y)
        local t = 1 + abs( clip_sym(y) )
        return 1 / (2 * t * t)
    end,
    msef       = 0.5,
}

Act.threshold_symmetric = {
    activation = function(self, x) return x > 0 and 1 or -1                end,
    derivative = function(self, y) error('Can\'t be used in training')     end, 
    msef       = 0.5,
}

return Act
