local Data   = {}
local DataMT = {__index = Data}

function Data.new(n_inp, n_out)
    local self = {}
    setmetatable(self, DataMT)
    
    self.n_inp      = n_inp
    self.n_out      = n_out
    self.n_total    = n_inp + n_out
    
    return self
end

function Data:add(tbl)
    for i, j in ipairs(tbl) do self[#self + 1] = j end
end

local function fmt(val)
    if math.floor(val) == val then return tostring(val) end
    return string.format('%.9f', val):gsub(',', '.')
end

function Data.new_from_file(path)
    local file = io.open(path)
    local n_rows, n_inp, n_out = file:read('*n'), file:read('*n'), file:read('*n')
    local n_total = n_inp + n_out
    
    local self = Data.new(n_inp, n_out)
    
    for i = 1, n_total do
        for r = 1, n_rows do
            self[#self + 1] = file:read('*n')
        end
    end
    
    file:close()
    
    return self
end

function Data:save_to_file(path)
    local f = io.open(path, 'w')
    
    f:write( string.format('%d %d %d\n', self:length(), self.n_inp, self.n_out) )
    
    local i = 1
    
    for s = 1, self:length() do
        for v = 1, self.n_inp do
            if v > 1 then f:write(' ') end
            f:write( fmt( self[i] ) )
            i = i + 1
        end
        
        f:write('\n')
        
        for v = 1, self.n_out do
            if v > 1 then f:write(' ') end
            f:write( fmt( self[i] ) )
            i = i + 1
        end
        
        f:write('\n')
    end
    
    f:close()
end

function Data:add(tbl)
    for i, j in ipairs(tbl) do
        self[#self + 1] = j
    end
end

function Data:length()
    return #self / self.n_total
end

function Data:get_index(row)
    return self.n_total * (row - 1) + 1, self.n_total - 1
end

return Data
